function[X,Y]=ALS_algorithm(G,M,n_dim,labels,vis)

## /**********************************************************************
##   Copyright 2013, Hanchen Xiong 
##   email: hanchen.xiong@uibk.ac.at
##
##   This is free software: you can redistribute it and/or modify
##   it under the terms of the GNU Lesser General Public License as published by
##   the Free Software Foundation, either version 3 of the License, or
##   (at your option) any later version.
##
##   This software is distributed in the hope that it will be useful,
##   but WITHOUT ANY WARRANTY; without even the implied warranty of
##   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##   GNU Lesser General Public License for more details.
##
##   You can check the GNU Lesser General Public License in <http://www.gnu.org/licenses/>.
##
## ***********************************************************************/

%------------------------------------------------------------------------------
%            author: Hanchen Xiong, University of Innsbruck
%------------------------------------------------------------------------------



J=size(G,2);                    % number of variables
N=size(G{1,1},1);               % number of objects
Y=cell(1,J);                    % category quantification
X=rand(N,n_dim);                % randomly initialize X
old_objective=0;
% begin iteration 
while 1
  % update y_j
  for j=1:J
    m_j=M{1,j};
    g_j=G{1,j};
    D_j=g_j'*m_j*g_j;
    y_j=inv(D_j)*g_j'*X;
    Y{1,j}=y_j;
  end; 
  % update X
  m_star=zeros(N,N);            % sum of m_j 
  sum_gy=zeros(N,n_dim);
  for j=1:J
    g_j=G{1,j};
    m_j=M{1,j};
    y_j=Y{1,j};
    m_star=m_star+M{1,j};
    sum_gy=sum_gy+g_j*y_j;
  end;  
  X=inv(m_star)*sum_gy;
  % normalize X
  X=Gram_Schmidt_process(X);
  objective=0;
  for j=1:J
    y_j=Y{1,j};
    m_j=M{1,j};
    g_j=G{1,j};
    D_j=g_j'*m_j*g_j;
    objective=objective+trace(y_j'*D_j*y_j);
  end;
  if (abs(objective-old_objective)<0.000001)
    break;
  else
    old_objective=objective;
  end;
end;
if vis==1
  figure;
  set(gcf,'position',[200,100,650,500]);               
  scatter3(X(:,1),X(:,2),X(:,3),20,'filled','blue');
  view(60,40);
  title('Object Scores in 3D Space');
  figure;
  set(gcf,'position',[200,100,650,500]);        
  markers = {'x','o','*','s','p','h','+','d','^','v','>','<','.'};
  colors={[0 0 1],[1 0 0],[0 1 0],[0 0 0],[1 0 1],[0 1 1],[1 1 0]};
  legend_text=cell(0,0);
  for j=1:J
    y_j=Y{1,j};
    num_category_j=size(y_j,1);
    if num_category_j==2
        scatter3(y_j(1,1),y_j(1,2),y_j(1,3),150,markers{j},'blue','linewidth', 2);
        hold on;
        scatter3(y_j(2,1),y_j(2,2),y_j(2,3),150,markers{j},'red','linewidth', 2);
        hold on;
        legend_text=[legend_text;[labels{j},'-']];
        legend_text=[legend_text;[labels{j},'+']];
   else
     for num_j=1:num_category_j 
        scatter3(y_j(num_j,1),y_j(num_j,2),y_j(num_j,3),150,markers{j}, 'MarkerEdgeColor',colors{1,num_j},'linewidth', 2);
        hold on;
        legend_text=[legend_text;[labels{j}, num2str(num_j)]];
     end;
   end; 
  end;
  view(60,40);
  set(gca,'linewidth',2);
  AX= legend(legend_text);
  LEG=findobj(AX,'type','text');
  set(LEG,'fontsize',12); 
  title('Category Quantification','fontsize',14);
end;
end
